/*
 * main.c
 *
 * Copyright (c) 2004 Atmark Techno, Inc.  All Rights Reserved.
 *
 */

#include "xparameters.h"
#include <ctype.h>

#include "srec.h"
#include "flash.h"
#include "medium.h"

#define PROG_NAME "BBoot - SUZAKU's first stage bootloader and S-Record downloader"
#define MAX_BUFFER_SIZE 64

/* Start and end of SDRAM */
#define DRAM_START    XPAR_SDRAM_CONTROLLER_BASEADDR
#define LOAD_ADDRESS  (XPAR_SDRAM_CONTROLLER_HIGHADDR - 0xFFFFF)

static int download(void)
{
	char buffer[MAX_BUFFER_SIZE];
	char *ptr1, *ptr2;
    unsigned int total_len;

	while(1)
	{
		get_line(buffer, MAX_BUFFER_SIZE);

		/* Chew through any leading white space */
		ptr1=buffer;
		while(*ptr1&& isspace(*ptr1))
			ptr1++;
	
		/* And truncate any trailing white space */
		ptr2=ptr1;
		while(*ptr2 && !isspace(*ptr2))
			ptr2++;
		*ptr2='\0';


		total_len += srec_decode_line(ptr1);
		put_char('.');
	}

	return 0;
}

static void copy_flash_to_dram()
{
    int i;
    volatile unsigned short *dst = (unsigned short *)LOAD_ADDRESS;

	for(i=0; i<FLASH_SIZE; i+=2)
	{
        *dst++ = flash_read(FLASH_START + i);
	}
	return ;
}

static int is_autoboot_mode(void)
{
	return *(volatile long *)(0xffffa000) & 0x4;
}

#define JUMP(addr) { ((void(*)(void))(addr))(); }

void second_bootloader(void)
{
	copy_flash_to_dram();
	myprint("Switching to 2nd stage bootloader..\n\r");
	JUMP(LOAD_ADDRESS); /* Never returns */
}

int main(void)
{
	
	if (is_autoboot_mode()) {
		second_bootloader();
	}
	else {
		char key;

		myprint("\n\r\n\rWelcome to " PROG_NAME "\n\r\n\r");
		myprint("hit Enter key to activate second stage bootloader or\n\r");
		myprint("hit 'S' key to download S-Record\n\r");
		while (1) {
			key = get_char();
			if ((key == 's') || (key == 'S')) {
				myprint("Erasing Flash...");
				flash_erase();
				myprint("Done\n\r");
				myprint("Start sending S-Record!!\n\r");
				download(); /* never returns */
			}
			else if (key == '\r') {
				second_bootloader(); /* never returns */
			}
			else {
				myprint("Please hit 'S' or Enter key\n\r");
			}
		}
	}

    return 0;
}
